/***********************************************************
*  SAM7SExx_DevMaster - Init of external DOGM163 on NPCS3  *
*                                                          *
*    generated by Frank Goetze  -   www.embedded-os.de     *
***********************************************************/
#include "../inc/OS_Def_AT91SAM7xxx.h"

/***********************************************************
*         user-functions prototypes (EA DOG-M 163)         *
***********************************************************/
void   TLCD_Init(void);                                    // init the used SPI and the LCD
void   TLCD_Reset(void);                                   // reset the LCD
void   TLCD_OnOff(U08 o);                                  // switch the display ON / OFF
void   TLCD_Contrast(U08 c);                               // set the contrast 0..63
void   TLCD_ClearScreen(void);                             // clear screen & home
void   TLCD_DBheight(U08 line);                            // define double height lines
void   TLCD_PutChar(U08 w);                                // write a char onto LCD
void   TLCD_PutString(U08 OS_HUGE *s);                     // write a string onto LCD
void   TLCD_GotoXY(U08 x, U08 y);                          // set the cursor to 0..15, 0..2
void   TLCD_GotoHome(void);                                // set cursor home (0,0)
void   TLCD_CreateCG(U08 addr, U08 OS_HUGE *array);        // crate own character in CGRAM


/***********************************************************
*                    definitions                           *
***********************************************************/
#define CPU_CRISTAL_CLK  18432000UL                        // processor crystal-clock in Hz

#define AT91S_SPI_NPCS3       AT91S_PB22                   // /MCS3 PIO-pin
#define AT91S_SPI_MISO        AT91S_PIO_PA12               // MISO  PIO-pin
#define AT91S_SPI_MOSI        AT91S_PIO_PA13               // MOSI  PIO-pin
#define AT91S_SPI_SPCK        AT91S_PIO_PA14               // SCK   PIO-pin
#define AT91S_LCD_A0          AT91S_PB29                   // A0 or RS (cmd/data) PIO-pin
#define AT91S_LCD_RES         AT91S_PB31                   // LCD-Reset PIO-pin
#define LCDport_CS_ON         AT91S_PIOB_CODR = AT91S_SPI_NPCS3;
#define LCDport_CS_OFF        AT91S_PIOB_SODR = AT91S_SPI_NPCS3;
#define LCDport_A0_LOW        AT91S_PIOB_CODR = AT91S_LCD_A0;
#define LCDport_A0_HIGH       AT91S_PIOB_SODR = AT91S_LCD_A0;

/***********************************************************
*                       globals                            *
***********************************************************/
// in the following init-sequence, the leading 30,30,30,20 are needed !
OS_CONST U08 OS_ROMDATA TLCD_InitParams[] = { 0x30,0x30,0x30,0x20,0x39,0x15,0x55,0x6E,0x72,0x38,0x0f,0x01,0x06 };

OS_CONST U08 OS_ROMDATA TLCD_CG_AccuFull[]   = {0x06,0x0F,0x0F,0x0F,0x0F,0x0F,0x0F,0x00};
OS_CONST U08 OS_ROMDATA TLCD_CG_AccuOk[]     = {0x06,0x09,0x09,0x0F,0x0F,0x0F,0x0F,0x00};
OS_CONST U08 OS_ROMDATA TLCD_CG_AccuEmpty[]  = {0x06,0x09,0x09,0x09,0x09,0x09,0x0F,0x00};
OS_CONST U08 OS_ROMDATA TLCD_CG_AccuLoad[]   = {0x06,0x09,0x0D,0x0F,0x0B,0x09,0x0F,0x00};
OS_CONST U08 OS_ROMDATA TLCD_CG_copyRight1[] = {0x07,0x08,0x13,0x14,0x14,0x13,0x08,0x07};
OS_CONST U08 OS_ROMDATA TLCD_CG_copyRight2[] = {0x00,0x10,0x08,0x08,0x08,0x08,0x10,0x00};

/***********************************************************
*       set max possible SPI baudrate and return this      *
***********************************************************/
U32 OS_setSPI_BR(U32 maxclk)
{
    U32 s_clksrc;
    U08 i, m;

    s_clksrc = CPU_CRISTAL_CLK;                                        // set start-clk (cristal-clk)
    AT91S_SPI_MR &= ~AT91S_SPI_MR_FDIV;                                // clear FDIV at calc-startup
    if((AT91S_PMC_MCKR & AT91S_PMC_PLLCLK) == AT91S_PMC_PLLCLK) {      // if PLL is enabled ..
        if((AT91S_CKGR_PLLR & AT91S_CKGR_DIV) > 0) {                   // .. if PLL-divisor set ..
            s_clksrc /= (AT91S_CKGR_PLLR & AT91S_CKGR_DIV);            // div clk with PLL-div
        }
        s_clksrc *= (((AT91S_CKGR_PLLR & AT91S_CKGR_MUL) >> 16) + 1);  // mul clk with PLL-multiplier
    }
    s_clksrc /= (0x0001UL << ((AT91S_PMC_MCKR & AT91C_PMC_PRES_CLK_M) >> 2));  // div clk with MCK-div
    for(i=1, m=1; m<32; i=1, m=32) {                                   // search lowest clk-div for SPI-speed (highest possible SPI-speed)
        while(((s_clksrc / (i * m)) > maxclk) && (i > 0)) i++;
        if(i > 0) break;
    }
    if(m > 1) AT91S_SPI_MR |= AT91S_SPI_MR_FDIV;           // if hole SPI-div greater 255 set FDIV for enable pre-div(32)
    AT91S_SPI_CSR->s[3] = (AT91S_SPI_CSR->s[3] & ~AT91S_SPI_CSR_SCBR_M) | (i << 8);
    return(s_clksrc / (i * m));                            // set SPI-div and return the used SPI-speed in Hz
}

/***********************************************************
*                     simple delay                         *
***********************************************************/
void LCDport_Delay(U32 d)
{
    d *= 10;
    while(d--) { _NOP(); _NOP(); _NOP(); }                 // the delay-times are importend on DOG-M displays !!!
}

/***********************************************************
*                     reset the LCD                        *
***********************************************************/
void TLCD_Reset(void)
{
    AT91S_PIOB_CODR = AT91S_LCD_RES;
    LCDport_Delay(100);
    AT91S_PIOB_SODR = AT91S_LCD_RES;
}

/***********************************************************
*               write a char to display                    *
***********************************************************/
void TLCD_PutChar(U08 w)
{
    LCDport_CS_ON;
    LCDport_Delay(5);
    AT91S_SPI_TDR = w;                                     // write char and send
    while(!(AT91S_SPI_SR & AT91S_SPI_SR_TDRE));            // wait up to tx-empty
    LCDport_Delay(50);
    LCDport_CS_OFF;
}

/***********************************************************
*               write a string to display                  *
***********************************************************/
void TLCD_PutString(U08 OS_HUGE *s)
{
    U32 i;

    i = 0; while(s[i] != 0) TLCD_PutChar(s[i++]);
}

/***********************************************************
*                      write a CMD                         *
***********************************************************/
void TLCDport_Cmd(U08 w)
{
    LCDport_A0_LOW;
    TLCD_PutChar(w);
    LCDport_A0_HIGH;
}

/***********************************************************
*            set cursor position (0..15, 0..2)             *
***********************************************************/
void TLCD_GotoXY(U08 x, U08 y)
{
    if(x > 16) x = 16;
    if(y > 2)  y = 2;
    TLCDport_Cmd(0x80 | (0x10 * y) | x);
}

/***********************************************************
*              set cursor position to home                 *
***********************************************************/
void TLCD_GotoHome(void)
{
    TLCDport_Cmd(0x02);
}

/***********************************************************
*                     clear display                        *
***********************************************************/
void TLCD_ClearScreen(void)
{
    TLCDport_Cmd(0x01);
    LCDport_Delay(500);
}

/***********************************************************
*                create special character                  *
***********************************************************/
void TLCD_CreateCG(U08 addr, U08 OS_HUGE *array)
{
    U08 i;
    
    addr = 0x40 | (addr << 3);                             // (01aa-aiii) "i" is autoincremented internally
    TLCDport_Cmd(addr);                                    // set address to CG-RAM
    for(i = 0; i < 8; i++) {
        TLCD_PutChar(array[i]);
    }                                                      // WARNING! following line is needed but resets the cursor-position ...
    TLCDport_Cmd(0x80);                                    // ... to "0", since we could not read the last valid one !
}

/***********************************************************
*                 double height On/Off                     *
***********************************************************/
void TLCD_DBheight(U08 line)
{
    if(line > 2) line = 0;
    if(line) {
        TLCDport_Cmd(0x2E);                                // 1 = upper two lines as one
        TLCDport_Cmd(0x10 | ((line == 1)?8:0));            // 2 = lower two lines as one
        TLCDport_Cmd(0x2C);
        return;
    }
    TLCDport_Cmd(0x28);                                    // 0 = normal three lines
}

/***********************************************************
*              sets the contrast of display                *
***********************************************************/
void TLCD_Contrast(U08 c)
{
    c &= 0x3F;
    TLCDport_Cmd(0x39);                                    // switch command table
    TLCDport_Cmd(0x54 | (c >> 4));
    TLCDport_Cmd(0x70 | (c & 0x0F));
    TLCDport_Cmd(0x38);                                    // back to command table 0
}

/***********************************************************
*               sets the display ON or OFF                 *
***********************************************************/
void TLCD_OnOff(U08 o)
{
    o = (o > 0)?4:0;
    TLCDport_Cmd(0x0B | o);
}

/***********************************************************
*                 SPI port for LCD init                    *
***********************************************************/
TLCDport_InitSPI(void)
{
    AT91S_PIOB_PER       = AT91S_SPI_NPCS3 | AT91S_LCD_A0 | AT91S_LCD_RES;    // enable GPIO of /CS & A0 & Reset
    AT91S_PIOB_SODR      = AT91S_SPI_NPCS3 | AT91S_LCD_A0 | AT91S_LCD_RES;    // set all high
    AT91S_PIOB_OER       = AT91S_SPI_NPCS3 | AT91S_LCD_A0 | AT91S_LCD_RES;    // output enable
    AT91S_PMC_PCER       = (1 << AT91S_ID_PIOB);                              // enable clock on PIO-B

    AT91S_PIO_PDR        = AT91S_SPI_MISO | AT91S_SPI_MOSI | AT91S_SPI_SPCK;  // disable from PIO-mode
    AT91S_PIO_ASR        = AT91S_SPI_MISO | AT91S_SPI_MOSI | AT91S_SPI_SPCK;  // enable bit-functions
    AT91S_PMC_PCER       = (1 << AT91S_ID_SPI);                               // enable clock on SPI-interface
    AT91S_SPI_CR         = AT91S_SPI_CR_DIS;                                  // disable SPI
    AT91S_SPI_CR         = AT91S_SPI_CR_EN | AT91S_SPI_CR_SWRST;              // enable & reset SPI
    AT91S_SPI_MR         = AT91S_SPI_MR_CS3 | AT91S_SPI_MR_MODFDIS | AT91S_SPI_MR_MSTR;       // CS3 + fault-off + (fix) + master
    AT91S_SPI_CSR->s[3]  = AT91S_SPI_CSR_SCBR_L | AT91S_SPI_CSR_BITS_8 | AT91S_SPI_CSR_CPOL;  // low-speed +  8bit + clk-pol (mode 3)
    AT91S_SPI_CR         = AT91S_SPI_CR_EN;                                   // enable SPI
    OS_setSPI_BR(2000000);                                                    // set max 2MHz
}

/***********************************************************
*                  LCD init (EA DOG-M 163)                 *
***********************************************************/
void TLCD_Init(void)
{
    U08 i;

    TLCDport_InitSPI();                                    // init SPI
    TLCD_Reset();                                          // reset LCD
    LCDport_Delay(500);                                    // send init-commands
    i = 0; while(i < sizeof(TLCD_InitParams)) TLCDport_Cmd(TLCD_InitParams[i++]);
    LCDport_Delay(500);                                    // wait
}

/************************* END ***************************/




/***************************************************************************
****************************************************************************
* TEST MAIN                                                                *
****************************************************************************
***************************************************************************/
void main(void)
{
    TLCD_Init();
    TLCD_PutString("testpattern for clearing next");
    TLCD_ClearScreen();
    TLCD_GotoXY(7,2);
    TLCD_PutString("test");
    TLCD_CreateCG(0, (U08 OS_HUGE *)TLCD_CG_AccuFull);
    TLCD_CreateCG(1, (U08 OS_HUGE *)TLCD_CG_AccuOk);
    TLCD_CreateCG(2, (U08 OS_HUGE *)TLCD_CG_AccuEmpty);
    TLCD_CreateCG(3, (U08 OS_HUGE *)TLCD_CG_AccuLoad);
    TLCD_CreateCG(4, (U08 OS_HUGE *)TLCD_CG_copyRight1);
    TLCD_CreateCG(5, (U08 OS_HUGE *)TLCD_CG_copyRight2);
    TLCD_PutChar(0);                                       // cursor was auto-set to home by "TLCD_CreateCG(..)"
    TLCD_PutChar(1);
    TLCD_PutChar(2);
    TLCD_PutChar(3);
    TLCD_PutChar(4);
    TLCD_PutChar(5);
    TLCD_PutChar(' ');                                     // test normal chars
    TLCD_PutChar('A');
    TLCD_OnOff(0);
    TLCD_OnOff(1);
    TLCD_GotoHome();
    TLCD_PutString("overwrite");
    TLCD_ClearScreen();
    TLCD_DBheight(1);
    TLCD_PutString("  double upper  ");
    TLCD_PutString("  normal lower  ");
    TLCD_ClearScreen();
    TLCD_DBheight(2);
    TLCD_PutString("  normal upper  ");
    TLCD_PutString("  double lower  ");
    TLCD_DBheight(0);
    TLCD_GotoXY(0,1);
    TLCD_PutString("     normal     ");
    TLCD_Contrast(58);
    TLCD_Contrast(4);
    TLCD_Contrast(16);
    TLCD_ClearScreen();
    TLCD_PutString(" embedded-os.de ");
    TLCD_PutString("  SAM7SExx-     ");
    TLCD_PutString("     DevMaster ");
    while(1);
}


